#pragma once
#include "../../YoggieEngine/src/YoggieEngine.h"
#include "EditorWindow.h"
#include "AssetManagement/AssetRegistry.h"

const char* hidden_extensions [] {
    ".exe",
    ".pdb",
    ".idb",
    ".dll",
    ".ini"
};

class AssetFinder : public EditorWindow {
public:
    AssetFinder () : EditorWindow("Assets") {}
    AssetFinder(const std::filesystem::path& projectdirectory) : EditorWindow("Assets")
    {
        for (auto& dir_entry : std::filesystem::directory_iterator(projectdirectory)) {
            auto filepath = dir_entry.path();

            if (dir_entry.is_directory() || dir_entry.is_symlink() || dir_entry.is_socket())
                continue;

            bool has_hidden_extension = false;
            for (auto hide : hidden_extensions) {
                if (filepath.extension() == hide)
                {
                    has_hidden_extension = true;
                    break;
                }
            }

            if (has_hidden_extension)
                continue;

            Asset asset(filepath.filename().string().c_str());
            asset.setFilPath(filepath.string());
            spdlog::info("Created asset: {0}", asset.GetName());
            files.push_back(asset);
        
        }

    }

    void Draw() override {

        //assetIcon = YoggieEngine::Texture("rsc/AssetIcon.png");
        ImGui::DragInt("IconSize", &iconSize, 1, 30, 90);
        ImGui::DragInt("Maximum Columns", &maxColumns, 1, 1, 6);
        if (ImGui::BeginTable("##resources", 3))
        {
            ImGui::PushStyleColor(ImGuiCol_Button, ImVec4(0.f, 0.f, 0.f, 0.f));
            ImGui::PushStyleColor(ImGuiCol_ButtonActive, ImVec4(0.f, 0.f, 0.f, 0.f));
            ImGui::PushStyleColor(ImGuiCol_ButtonHovered, ImVec4(1.f, 1.f, 1.f, 0.2f));


            
            int row = 0;
            int column = 0;
            for (auto& asset : files ) {
                if (column % 3 == 0) {
                    ImGui::TableNextRow();
                    column = 0;
                    row++;
                }

                ImGui::TableSetColumnIndex(column);

                ImGui::ImageButton(
                    (ImTextureID)assetIcon.GetID(),
                    ImVec2{ (float)iconSize, (float)iconSize });
                ImGui::Text(asset.GetName(), row);



                column++;
            }
            

            
            ImGui::PopStyleColor(3);
            ImGui::EndTable();
            const GLuint textures[2]{ assetIcon.GetID(), folderIcon.GetID() };
            glDeleteTextures(2, textures);

        }
    }

private:
    std::vector <Asset> files = std::vector<Asset>();

    int iconSize = 60;
    int maxColumns = 3;

    YoggieEngine::Texture folderIcon;
    YoggieEngine::Texture assetIcon;

};

