#include "Viewport.h"

Viewport::Viewport(YoggieEngine::Scene& scene, YoggieEngine::Entity& selected) :
    EditorWindow("SceneView"),
    renderer(YoggieEngine::RendererConfig{ 1200, 700, glm::vec3(0), true })
{
    CurrentScene = &scene;
    this->selected = &selected;
}

void Viewport::Draw() {

    auto group = CurrentScene->getReg().view<YoggieEngine::TransformComponent, YoggieEngine::Render3DComponent>();
    group.each([&](auto enity, YoggieEngine::TransformComponent& t, YoggieEngine::Render3DComponent& renderComponent) {
        renderer.Submit(renderComponent, t);
        });
    
    isFocused = ImGui::IsWindowFocused();

    renderer.Render(*CurrentScene);

    ImGui::Image(
        (void*)(intptr_t)renderer.getCurrentFrameBuffer().GetColourAttachment(),
        ImVec2{ (float)ImGui::GetWindowWidth(),(float)ImGui::GetWindowHeight() }
    );


    

    ImGuizmo::Enable(true);
    ImGuizmo::SetOrthographic(false);
    ImGuizmo::SetDrawlist();

    float windowWidth = (float)ImGui::GetWindowWidth();
    float windowHeight = (float)ImGui::GetWindowHeight();
    ImGuizmo::SetRect(ImGui::GetWindowPos().x, ImGui::GetWindowPos().y, windowWidth, windowHeight);

    const auto& ProjMatrix = camera.projection;
    glm::mat4& cameraView = glm::mat4(1.0f);//glm::inverse(glm::translate(glm::mat4(1.0f) , cam.Position) * glm::toMat4(glm::quat(cam.Rotation)) );

    ImGuizmo::DrawGrid(glm::value_ptr(cameraView), glm::value_ptr(ProjMatrix), glm::value_ptr(cameraDelta), 100.0f);
    ImGuizmo::ViewManipulate(glm::value_ptr(cameraView), 1, ImGui::GetWindowPos(), {90,90}, 0x22CCCCCC);


    if (selected == nullptr)
        return;

    if (selected->isValid()) {
      
        auto& tc = selected->GetComponent<YoggieEngine::TransformComponent>();
        glm::mat4 transform = tc.GetTransform();

        ImGuizmo::Manipulate(
                    glm::value_ptr(cameraView),
            glm::value_ptr(ProjMatrix),
            ImGuizmo::TRANSLATE, ImGuizmo::LOCAL,
            glm::value_ptr(transform), nullptr, nullptr);
    }

   
}