#pragma once
#include "GLFW/glfw3.h"
#include <vector>
#include "../PerfCounter.h"

#include "Renderable.h"
#include "Memory/Framebuffer.h"
#include "../Scene/Components.h"
#include"../Scene/Scene.h"
#include "Graphics/Primitives/DrawCommand.h"
#include "Primitives/Camera.h"

namespace YoggieEngine {

	struct RendererConfig {
		int ScreenWidth, ScreenHeight;
		glm::vec3 ClearColor;
		bool DepthTest; 
	};

	class Renderer {
	public:
		Renderer(RendererConfig& config);
		~Renderer();


		void Submit(Render3DComponent& renderComponent, TransformComponent& transform); // Collects DrawCommands
		void Render(Scene&);	// Draw to screen (using drawCall structs)
	
		void setCurrentFrameBuffer(const Framebuffer& fb);
		void setClearColor(const glm::vec3& ClearColor);

		void SetMainCamera(const Camera& camera) { MainCamera = camera;  }

		Framebuffer& getCurrentFrameBuffer() { return m_framebuffer; }
	private:
		void GeometryPass();
		void ForwardGeometryPass();
		void SkyboxPass();
		void lightingPass(Scene& scene);
		void BlendingPass();
		void PostProcessing();
		void CopyGBuffer();


	private:
		Framebuffer m_framebuffer;
		int width, height;
		glm::vec3 m_clearColor;
		bool m_depthTest;
		std::vector<DrawCommand> commands;
		Camera MainCamera;

        unsigned int skyboxVAO = 0;
		CubeMap sky;

		Shader forwardShader;

		// deferred rending parameters
		unsigned int gBuffer, gPosition, gNormal, gColorSpec, gDepth;
		Shader lightingPassShader;
		Shader gBufferShader;
		Shader SkyboxShader;
		Shader postProcessingShader;


		// blending
		Shader BlendingShader;
	};
}
